//Definition of variables
var years;
var months;
var days;
var hours;
var minutes;
var seconds;
var name;

var cache = CacheService.getScriptCache();
var SHEET_NAME = cache.get('sheetName');

/*!
 * brief Sets time variables to actual time.
 *
 */
function getTime()
{
  var d = new Date();

  years = d.getFullYear();
  months = (d.getMonth() + 1);
  days = d.getDate();
  hours = d.getHours();
  minutes = d.getMinutes();
  seconds = d.getSeconds();
  time = d.getTime();

  if (months < 10) {months = "0" + months;}
  if (days < 10) {days = "0" + days;}
  if (hours < 10) {hours = "0" + hours;}
  if (minutes < 10) {minutes = "0" + minutes;}
  if (seconds < 10) {seconds = "0" + seconds;}
}

/*!
 * brief Creates name for new log sheet.
 *
 */
function getName()
{
  getTime();
  
  name = "";

  name += days;
  name += "_";
  name += months;
  name += "_";
  name += years;
  name += "__";
  name += hours;
  name += "-";
  name += minutes;
  name += "-";
  name += seconds;
}

/*!
 * brief Creates new log sheet.
 *
 */
function changeSheet()
{
  getName();
  var doc = SpreadsheetApp.openById(SCRIPT_PROP.getProperty("key"));
  doc.insertSheet(name);
  var sheet = doc.getSheetByName(name);
  sheet.appendRow(["Timestamp", "date", "time", "temperature_1", "temperature_2", "temperature_3", "temperature_4", "relay", "Info_message"]);
  sheet.hideColumns(1);
  cache.put('sheetName', name);
  SHEET_NAME = name;
}

var SCRIPT_PROP = PropertiesService.getScriptProperties(); // new property service

/*!
 * brief Captures a POST request.
 *
 */
function doPost(e){

  return handleResponse(e);
}

/*!
 * brief POST request processing.
 * This feature was taken from https://gist.github.com/alkrauss48/39b27802a080ce436015 and modified for the needs of this script. 
 *
 */
function handleResponse(e) {

  var lock = LockService.getPublicLock();
  lock.waitLock(30000);  // wait 30 seconds before conceding defeat.

  try {
    // next set where we write the data - you could write to multiple/alternate destinations
    var doc = SpreadsheetApp.openById(SCRIPT_PROP.getProperty("key"));
    var sheet = doc.getSheetByName(SHEET_NAME);

    // we'll assume header is in row 1 but you can override with header_row in GET/POST data
    var headRow = e.parameter.header_row || 1;
    var headers = sheet.getRange(1, 1, 1, sheet.getLastColumn()).getValues()[0];
    var nextRow = sheet.getLastRow()+1; // get next row
    var row = [];
    // loop through the header columns
    for (i in headers){
      if (headers[i] == "Timestamp"){ // special case if you include a 'Timestamp' column
        row.push(new Date());
      } else { // else use header name to get data
        row.push(e.parameter[headers[i]]);
      }
    }
    
    if (row[2] === "x")
    {
      changeSheet();
    }
    
    else
    {
    
    // more efficient to set values as [][] array than individually
    sheet.getRange(nextRow, 1, 1, row.length).setValues([row]);
    // return json success results
    return ContentService
          .createTextOutput(JSON.stringify({"result":"success", "row": nextRow}))
          .setMimeType(ContentService.MimeType.JSON);
    }
          
  } catch(e){
    // if error return this
    changeSheet();
    return ContentService
          .createTextOutput(JSON.stringify({"result":"error", "error": e}))
          .setMimeType(ContentService.MimeType.JSON);
  } finally { //release lock
    lock.releaseLock();
  }
}

function setup() {
    var doc = SpreadsheetApp.getActiveSpreadsheet();
    SCRIPT_PROP.setProperty("key", doc.getId());
}