(function() {
  if (window.jQuery === undefined) {
    console.error("FMNG_Settings.js requires jQuery");
    return;
  }

  var $ = window.jQuery;
  var _connMgr, _symMgr;

  (function() {
    const CONN_MOD = {
      RS232: "RS232",
      PLUGIN: "Plug-in module"
    };
    const BAUD_RATES = [1200, 2400, 4800, 9600, 14400, 19200, 38400, 56000, 57600, 115200, 128000, 256000];
    const PLUGINS = {
      HCS: "FreeMASTER BDM Communication Plug-in (CortexM, PowerPC, HCS08/12, ColdFire)",
      EONCE: "FreeMASTER BDM JTAG/EOnCE Communication Plug-in (56F8xxx)",
      DAP: "FreeMASTER CMSIS DAP Communication Plug-in",
      PDBDM: "FreeMASTER Packet Driven BDM Communication Plug-in",
      JLINK: "FreeMASTER Segger/JLink Communication Plug-in",
      CAN: "FreeMASTER-over-CAN Communication Plug-in",
      LIN: "FreeMASTER-over-LIN Communication Plug-in"
    };

    /**
     * @class
     * @classdesc This widet is used by {@link FMNG_Settings Settings} to allow communication port setup.
     * @description Creates an instance of FMNG_SettingsComView.
     * @see {@link FMNG_Settings Settings}
     * @this {FMNG_SettingsComView}
     * @param {jQuery} element The host element.
     */
    this.FMNG_SettingsComView = function(element) {
      if (element) {
        this.render(element);
        this.registerEventHandlers();
        this.serialRadio.trigger("click");
      }
    };

    /**
     * Renders the settings communication view inside a passed host element.
     *
     * @this {FMNG_SettingsComView}
     * @param {jQuery} host The host element.
     */
    FMNG_SettingsComView.prototype.render = function(host) {
      this.host = host;
      this.comFieldset = $("<fieldset>");
      this.comLegend = $("<legend>").text("Communication");
      this.panelGroup = $("<div>").addClass("panel-group");
      this.namedConnPanel = $("<div>").addClass("panel panel-default");
      this.namedConnHeading = $("<div>").addClass("panel-heading");
      this.namedConnLabel = $("<label>").text("Named Connections");
      this.namedConnHeading.append(this.namedConnLabel);
      this.namedConnectionSelect = $("<select>").addClass("form-control");
      this.namedConnectionSelect.append("<option value='' selected disabled hidden>Choose here</option>");
      this.namedConnPanel
        .append(this.namedConnHeading)
        .append(this.namedConnectionSelect);
      this.serialPanel = $("<div>").addClass("panel panel-default");
      this.serialHeading = $("<div>").addClass("panel-heading radio");
      this.serialLabel = $("<label>");
      this.serialRadio = $("<input>").attr("type", "radio").attr("name", "com").val(CONN_MOD.RS232);
      this.serialLabel
        .append(this.serialRadio)
        .append(CONN_MOD.RS232);
      this.serialHeading.append(this.serialLabel);
      this.serialContent = $("<div>").addClass("panel-body");
      this.serialPort = $("<div>").addClass("col-sm-6");
      this.serialPortLabel = $("<label>").text("Port");
      this.serialPortInput = $("<input>").addClass("form-control").val("COMx");
      this.serialPort
        .append(this.serialPortLabel)
        .append(this.serialPortInput);
      this.serialBaudRate = $("<div>").addClass("col-sm-6");
      this.serialBaudRateLabel = $("<label>").text("Baud Rate");
      this.serialBaudRateInput = $("<input>").attr("list", "fmng_settings_baudrate").addClass("form-control").val(115200);
      this.serialBaudRateDataList = $("<datalist>").attr("id", "fmng_settings_baudrate");
      for (let option of BAUD_RATES) {
        let optionItem = $("<option>").val(option).text(option);
        this.serialBaudRateDataList.append(optionItem);
      }
      this.serialBaudRate
        .append(this.serialBaudRateLabel)
        .append(this.serialBaudRateInput)
        .append(this.serialBaudRateDataList);
      this.serialTimeoutHeading = $("<div>").addClass("panel-heading").text("Timeout");
      this.serialTimeoutContent = $("<div>").addClass("panel-body");
      this.serialInterByteTimeout = $("<div>").addClass("col-sm-12");
      this.serialInterByteTimeoutLabel = $("<label>").text("Inter Byte Timeout");
      this.serialInterByteTimeoutInput = $("<input>").addClass("form-control").val("40");
      this.serialInterByteTimeout
        .append(this.serialInterByteTimeoutLabel)
        .append(this.serialInterByteTimeoutInput);
      this.serialReadTimeoutConst = $("<div>").addClass("col-sm-6");
      this.serialReadTimeoutConstLabel = $("<label>").text("Read Timeout Constant");
      this.serialReadTimeoutConstInput = $("<input>").addClass("form-control").val("50");
      this.serialReadTimeoutConst
        .append(this.serialReadTimeoutConstLabel)
        .append(this.serialReadTimeoutConstInput);
      this.serialReadTimeoutMult = $("<div>").addClass("col-sm-6");
      this.serialReadTimeoutMultLabel = $("<label>").text("Read Timeout Multiplier");
      this.serialReadTimeoutMultInput = $("<input>").addClass("form-control").val("40");
      this.serialReadTimeoutMult
        .append(this.serialReadTimeoutMultLabel)
        .append(this.serialReadTimeoutMultInput);
      this.serialWriteTimeoutConst = $("<div>").addClass("col-sm-6");
      this.serialWriteTimeoutConstLabel = $("<label>").text("Write Timeout Constant");
      this.serialWriteTimeoutConstInput = $("<input>").addClass("form-control").val("50");
      this.serialWriteTimeoutConst
        .append(this.serialWriteTimeoutConstLabel)
        .append(this.serialWriteTimeoutConstInput);
      this.serialWriteTimeoutMult = $("<div>").addClass("col-sm-6");
      this.serialWriteTimeoutMultLabel = $("<label>").text("Write Timeout Multiplier");
      this.serialWriteTimeoutMultInput = $("<input>").addClass("form-control").val("40");
      this.serialWriteTimeoutMult
        .append(this.serialWriteTimeoutMultLabel)
        .append(this.serialWriteTimeoutMultInput);
      this.serialTimeoutContent
        .append(this.serialInterByteTimeout)
        .append(this.serialReadTimeoutConst)
        .append(this.serialReadTimeoutMult)
        .append(this.serialWriteTimeoutConst)
        .append(this.serialWriteTimeoutMult);
      this.serialSettingsGroup = $("<div>").addClass("panel-group");
      this.serialSettingsBase = $("<div>").addClass("panel panel-default");
      this.serailSettingsBaseContent = $("<div>").addClass("panel-body").css("display", "block");
      this.serailSettingsBaseContent
        .append(this.serialPort)
        .append(this.serialBaudRate);
      this.serialSettingsBase.append(this.serailSettingsBaseContent);
      this.serialSettingsTimeout = $("<div>").addClass("panel panel-default");
      this.serialSettingsTimeout
        .append(this.serialTimeoutHeading)
        .append(this.serialTimeoutContent);
      this.serialSettingsGroup
        .append(this.serialSettingsBase)
        .append(this.serialSettingsTimeout);
      this.serialContent.append(this.serialSettingsGroup);
      this.serialPanel
        .append(this.serialHeading)
        .append(this.serialContent);
      this.pgiPanel = $("<div>").addClass("panel panel-default");
      this.pgiHeading = $("<div>").addClass("panel-heading radio");
      this.pgiLabel = $("<label>");
      this.pgiRadio = $("<input>").attr("type", "radio").attr("name", "com").val(CONN_MOD.PLUGIN);
      this.pgiLabel
        .append(this.pgiRadio)
        .append(CONN_MOD.PLUGIN);
      this.pgiHeading.append(this.pgiLabel);
      this.pgiContent = $("<div>").addClass("panel-body");
      this.pgiModule = $("<div>").addClass("col-sm-12");
      this.pgiSelect = $("<select>").addClass("form-control");
      this.pgiSelect.append("<option value='' selected disabled hidden>Choose Plug-in module</option>");
      for (let module in PLUGINS) {
        let optionItem = $("<option>").val(module).text(PLUGINS[module]);
        this.pgiSelect.append(optionItem);
      }
      this.pgiModule.append(this.pgiSelect);
      this.pgiConString = $("<div>").addClass("col-sm-12");
      this.pgiConStringLabel = $("<label>").text("Connection String");
      this.pgiConStringInput = $("<input>").addClass("form-control");
      this.pgiConString
        .append(this.pgiConStringLabel)
        .append(this.pgiConStringInput);
      this.pgiContent
        .append(this.pgiModule)
        .append(this.pgiConString);
      this.pgiPanel
        .append(this.pgiHeading)
        .append(this.pgiContent);
      this.panelGroup
        .append(this.namedConnPanel)
        .append(this.serialPanel)
        .append(this.pgiPanel);
      this.comFieldset
        .append(this.comLegend)
        .append(this.panelGroup);
      this.host.append(this.comFieldset);
    };

    /**
     * Renders named connections defined in project file
     *
     * @param {Array<string>} commPorts List of named connection names.
     * @this {FMNG_SettingsComView}
     */
    FMNG_SettingsComView.prototype.renderNamedConnections = function(commPorts) {
      for (let commPortName of commPorts) {
        let option = $("<option>").val(commPortName).text(commPortName);
        this.namedConnectionSelect.append(option);
      }
    };

    /**
     * Registers default communication view event handlers.
     *
     * @this {FMNG_SettingsComView}
     */
    FMNG_SettingsComView.prototype.registerEventHandlers = function() {
      this.panelGroup.on("change", "input[name=\"com\"]", this.radioOnChange.bind(this));
      this.serialTimeoutHeading.on("click", () => this.serialTimeoutContent.toggle());
    };

    /**
     * Communication radio button change event handler.
     *
     * @this {FMNG_SettingsComView}
     * @param {Object} event JS radio change event.
     */
    FMNG_SettingsComView.prototype.radioOnChange = function(event) {
      let $target = $(event.target);
      if (!$target.prop("checked")) {
        return;
      }
      this.panelGroup.find(".panel-body").removeClass("active");
      $target.parent().parent().next().addClass("active");
    };

    /**
     * Gets connection string.
     *
     * @this {FMNG_SettingsComView}
     * @return {string} Connection string.
     */
    FMNG_SettingsComView.prototype.getConnectionString = function() {
      let radioValue = $("input[name=\"com\"]:checked"). val();
      switch (radioValue) {
        case CONN_MOD.RS232:
          return "RS232;port=" + this.serialPortInput.val() +
          ";speed=" + this.serialBaudRateInput.val() +
          ";tmoRI=" + this.serialInterByteTimeoutInput.val() +
          ";tmoRTM=" + this.serialReadTimeoutMultInput.val() +
          ";tmoRTC=" + this.serialReadTimeoutConstInput.val() +
          ";tmoWTM=" + this.serialWriteTimeoutMultInput.val() +
          ";tmoWTC=" + this.serialWriteTimeoutConstInput.val();
        case CONN_MOD.PLUGIN:
          let pluginModule = this.pgiSelect.val();
          let connString = this.pgiConStringInput.val();
          if (pluginModule && connString) {
            return pluginModule + ";" + connString;
          }
      }
      return "";
    };

    /**
     * Gets communication view data.
     *
     * @this {FMNG_SettingsComView}
     * @return {string} Connection string.
     */
    FMNG_SettingsComView.prototype.getData = function() {
      return this.getConnectionString();
    };

    /**
     * Sets connection string.
     *
     * @this {FMNG_SettingsComView}
     * @param {string} connStr Connection string.
     */
    FMNG_SettingsComView.prototype.setConnectionString = function(connStr) {
      let parts = connStr.split(";").map(s => s.trim());
      switch (parts[0]) {
        case CONN_MOD.RS232:
          let map = {};
          for (let i = 1; i < parts.length; ++i) {
            let subParts = parts[i].split("=").map(s => s.trim());
            map[subParts[0]] = subParts[1];
          }
          if (map.port !== undefined) {
            this.serialPortInput.val(map.port);
          }
          if (map.speed !== undefined) {
            this.serialBaudRateInput.val(map.speed);
          }
          if (map.tmoRI !== undefined) {
            this.serialInterByteTimeoutInput.val(map.tmoRI);
          }
          if (map.tmoRTM !== undefined) {
            this.serialReadTimeoutMultInput.val(map.tmoRTM);
          }
          if (map.tmoRTC !== undefined) {
            this.serialReadTimeoutConstInput.val(map.tmoRTC);
          }
          if (map.tmoWTM !== undefined) {
            this.serialWriteTimeoutMultInput.val(map.tmoWTM);
          }
          if (map.tmoWTC !== undefined) {
            this.serialWriteTimeoutConstInput.val(map.tmoWTC);
          }
          this.serialRadio.trigger("click");
          break;
        default:
          if (PLUGINS.hasOwnProperty(parts[0])) {
            this.pgiSelect.val(parts[0]);
            this.pgiConStringInput.val(connStr.substring(connStr.indexOf(";") + 1));
            this.pgiRadio.trigger("click");
          }
          break;
      }
    };

    /**
     * Sets communication view data.
     *
     * @this {FMNG_SettingsSymView}
     * @param {string} data Connection string.
     */
    FMNG_SettingsComView.prototype.setData = function(data) {
      this.setConnectionString(data);
    };

  })();

  (function() {

    /**
     * @class
     * @classdesc This widet is used by {@link FMNG_Settings Settings} to allow ELF & TSA parsing.
     * @description Creates an instance of FMNG_SettingsSymView.
     * @see {@link FMNG_Settings Settings}
     * @this {FMNG_SettingsSymView}
     * @param {jQuery} element The host element.
     */
    this.FMNG_SettingsSymView = function(element) {
      if (element) {
        this.render(element);
      }
    };

    /**
    * Renders the settings elf view inside a passed host element.
    *
    * @this {FMNG_SettingsSymView}
    * @param {jQuery} element The host element.
    */
    FMNG_SettingsSymView.prototype.render = function(host) {
      this.host = host;
      this.symFieldset = $("<fieldset>");
      this.symLegend = $("<legend>").text("Symbol Reader");
      this.elfFileInputLabel = $("<label>").text("ELF file");
      this.elfFileInput = $("<input>").addClass("form-control").attr("placeholder", "There's no elf associated with the current connection.");
      this.elfFileBtnGroupLabel = $("<label>").text("Press to parse ELF file");
      this.elfFileBtnGroup = $("<span>").addClass("btn-group");
      this.elfFileBtn = $("<button>").addClass("btn btn-primary").text("Load ELF");
      this.elfFileStatusSpan = $("<span>").text("Status");
      this.elfFileStatusBtn = $("<button>").addClass("btn btn-default").prop("disabled", true);
      this.elfFileStatusBtn.append(this.elfFileStatusSpan);
      this.elfFileBtnGroup
          .append(this.elfFileBtn)
          .append(this.elfFileStatusBtn);
      this.tsaBtnGroupLabel = $("<label>").text("Press to parse TSA table(s)");
      this.tsaBtnGroup = $("<span>").addClass("btn-group");
      this.tsaBtn = $("<button>").addClass("btn btn-primary").text("Load TSA");
      this.tsaStatusSpan = $("<span>").text("Status");
      this.tsaStatusBtn = $("<button>").addClass("btn btn-default").prop("disabled", true);
      this.tsaStatusBtn.append(this.tsaStatusSpan);
      this.tsaBtnGroup
          .append(this.tsaBtn)
          .append(this.tsaStatusBtn);
      this.symFieldset
        .append(this.symLegend)
        .append(this.elfFileInputLabel)
        .append(this.elfFileInput)
        .append("<br>")
        .append(this.elfFileBtnGroupLabel)
        .append("<br>")
        .append(this.elfFileBtnGroup)
        .append("<br>")
        .append(this.tsaBtnGroupLabel)
        .append("<br>")
        .append(this.tsaBtnGroup);
      this.host.append(this.symFieldset);
    };

    /**
     * Gets elf path.
     *
     * @this {FMNG_SettingsSymView}
     * @return {string} elf ELF file path.
     */
    FMNG_SettingsSymView.prototype.getElf = function() {
      return this.elfFileInput.val();
    };

    /**
     * Gets sym view data.
     *
     * @this {FMNG_SettingsSymView}
     * @return {string} data ELF file path.
     */
    FMNG_SettingsSymView.prototype.getData = function() {
      return this.getElf();
    };

    /**
     * Sets elf data.
     *
     * @this {FMNG_SettingsSymView}
     * @param {string} elf ELF file path.
     */
    FMNG_SettingsSymView.prototype.setElf = function(elf) {
      this.elfFileInput.val(elf);
    };

    /**
     * Sets sym view data.
     *
     * @this {FMNG_SettingsSymView}
     * @param {string} data ELF file path.
     */
    FMNG_SettingsSymView.prototype.setData = function(data) {
      this.setElf(data);
    };

  })();

  /**
   * @class
   * @classdesc Settings widget is a container with two tabs {@link FMNG_SettingsComView Communication View} and {@link FMNG_SettingsSymView Symbols View}.
   * To use FMNG_Settings in an application, folowing widgets need to be created first:
   * - {@link FMNG_ConnectionMgr} - manages connection to the target
   * - {@link FMNG_SymbolMgr} - manages target side aplication symbols
   *
   * Dependencies:
   * - {@link FMNG_SettingsComView Communication View}
   * - {@link FMNG_SettingsSymView Symbols View}
   * @description Creates an instance of FMNG_Settings.
   * @this {FMNG_Settings}
   * @param {jQuery}             element The host element.
   * @param {FMNG_ConnectionMgr} connMgr Connection manager widget.
   * @param {FMNG_SymbolMgr}     symMgr  Symbol manager widget.
   *
   * @example
   * $(document).ready(function() {
   *     pcm = new PCM(window.location.host, onSocketOpen);
   * });
   * function (onSocketOpen) {
   *     // Initialize connection manager widget
   *     connectionMgr = new FMNG_ConnectionMgr(pcm);
   *     // Initialize SymReader widget
   *     symbolMgr = new FMNG_SymbolMgr(pcm);
   *     settings = new FMNG_Settings($("#fmng_settings_container"), connectionMgr, symbolMgr);
   * };
   */
  this.FMNG_Settings = function(element, connMgr, symMgr) {
    _connMgr = connMgr;
    _symMgr = symMgr;
    if (element) {
      this.render(element);
      this.registerEventHandlers();
      _connMgr.getCommPorts().then(this.comView.renderNamedConnections.bind(this.comView));
    }
  };

  /**
   * Renders FMNG_Settings UI inside the provided host element.
   *
   * @this {FMNG_Settings}
   * @param {jQuery} host The host element.
   */
  FMNG_Settings.prototype.render = function(host) {
    this.host = host;
    this.settings = $("<div>").addClass("fmng_settings");
    this.settingsHeader = $("<div>").addClass("header");
    this.settings.append(this.settingsHeader);
    this.host.append(this.settings);
    this.comView = new FMNG_SettingsComView(this.addTab("Comm"));
    this.symView = new FMNG_SettingsSymView(this.addTab("Sym"));
    this.setActiveTab(this.settingsHeader.children().first());
  };

  /**
   * Registers default settings event handlers.
   *
   * @this {FMNG_Settings}
   */
  FMNG_Settings.prototype.registerEventHandlers = function() {
    this.settingsHeader.on("click", ".tablink", this.tabLinkOnClick.bind(this));
    this.comView.namedConnectionSelect.on("change", this.namedConnectionOnSelect.bind(this));
    this.symView.elfFileBtn.on("click", this.readElfOnClick.bind(this));
    this.symView.tsaBtn.on("click", this.readTsaOnClick.bind(this));
  };

  /**
   * Tab header on click event handler.
   *
   * @this {FMNG_Settings}
   * @param {Object} event Js click event.
   */
  FMNG_Settings.prototype.tabLinkOnClick = function(event) {
    this.settingsHeader.children(".tablink.active").removeClass("active");
    this.settings.children(".tabcontent.active").removeClass("active");
    this.setActiveTab($(event.target));
  };

  /**
   * Named connection select on change event handler.
   *
   * @this {FMNG_Settings}
   * @param {Object} event Js change event.
   */
  FMNG_Settings.prototype.namedConnectionOnSelect = function(event) {
    let portName = $(event.target).children("option:selected").text();
    _connMgr.getCommPortInfo(portName).then(conn => {
      this.comView.setData(conn.connectionString);
      this.symView.setData(conn.elf);
    });
  };

  /**
   * Read ELF button click event handler.
   *
   * @this {FMNG_Settings}
   */
  FMNG_Settings.prototype.readElfOnClick = async function() {
    this.symView.elfFileStatusSpan.empty().attr("class", "glyphicon glyphicon-transfer");
    if (await _symMgr.readELF(this.symView.getElf())) {
     this.symView.elfFileStatusSpan.empty().attr("class", "glyphicon glyphicon-ok");
   } else {
     this.symView.elfFileStatusSpan.empty().attr("class", "glyphicon glyphicon-remove");
    }
  };

  /**
   * Read TSA button click event handler.
   *
   * @this {FMNG_Settings}
   */
  FMNG_Settings.prototype.readTsaOnClick = async function() {
    this.symView.tsaStatusSpan.empty().attr("class", "glyphicon glyphicon-transfer");
    if (await _symMgr.readTSA()) {
      this.symView.tsaStatusSpan.empty().attr("class", "glyphicon glyphicon-ok");
    } else {
      this.symView.tsaStatusSpan.empty().attr("class", "glyphicon glyphicon-remove");
    }
  };

  /**
   * Returns connection string.
   *
   * @this {FMNG_Settings}
   * @return {string} Connection string.
   */
  FMNG_Settings.prototype.getConnectionString = function() {
    return this.comView.getConnectionString();
  };

  /**
   * Gets project settings.
   *
   * @this {FMNG_Settings}
   * @return {Object} Project settings data, containing *{@link FMNG_SettingsComView}* data.
   */
  FMNG_Settings.prototype.getData = function() {
    return {
      "connectionString": this.comView.getData(),
      "elf": this.symView.getData()
    };
  };

  /**
   * Sets project settings data.
   *
   * @this {FMNG_Settings}
   * @param {Object} data Project settings data, containing *{@link FMNG_SettingsComView}* data.
   */
  FMNG_Settings.prototype.setData = function(data) {
    this.comView.setData(data.connectionString);
    this.symView.setData(data.elf);
  };

  /**
   * Sets the active tab.
   *
   * @this {FMNG_Settings}
   * @param {jQuery} $tabHeader The active tab header.
   */
  FMNG_Settings.prototype.setActiveTab = function($tabHeader) {
    let $tabContent = this.settings.children("#" + $tabHeader.text());
    $tabHeader.addClass("active");
    $tabContent.addClass("active");
  };

  /**
   * Adds a new tab element to settings view.
   *
   * @this {FMNG_Settings}
   * @param {string} title The new tab title.
   * @return {jQuery} The new tab body element.
   */
  FMNG_Settings.prototype.addTab = function(title) {
    let tabHeader = $("<button>")
      .addClass("tablink")
      .text(title);
    let tabContent = $("<div>")
      .addClass("tabcontent")
      .attr("id", title);
    this.settingsHeader.append(tabHeader);
    this.settingsHeader.after(tabContent);
    return tabContent;
  };

  /**
   * Tooggles settings view visibility.
   *
   * @this {FMNG_Settings}
   */
  FMNG_Settings.prototype.toggle = function() {
    this.settings.toggle();
  };

}());
