//Definition of variables

var handle = 0;
var htmlHandle = 0;

var intervalHandle1 = 0;
var intervalHandle2 = 0;
var intervalHandle3 = 0;
var intervalHandle4 = 0;

var years;
var months;
var days;
var hours;
var minutes;
var seconds;
var time;

var temperature1;
var temperature2;
var temperature3;
var temperature4;
var current;
var relayState;

var disconnectionSrc = 0;

var old_temperature1 = 0;
var old_temperature2 = 0;
var old_temperature3 = 0;
var old_temperature4 = 0;

var old_old_temperature1 = 0;
var old_old_temperature2 = 0;
var old_old_temperature3 = 0;
var old_old_temperature4 = 0;

var old_relayState = false;
var old_disconnectionSrc = 0;

var defaultLimit1 = 50;
var defaultLimit2 = 60;
var defaultLimit3 = 70;
var defaultLimit4 = 80;
var defaultLimit5 = 5;

var lim1 = undefined;
var lim2 = undefined;
var lim3 = undefined;
var lim4 = undefined;
var lim5 = undefined;

var tempLine1 = [];
var tempLine2 = [];
var tempLine3 = [];
var tempLine4 = [];
var timeLine = [];

var testValue;
var oldTestValue;
var connectionState;
var failNumber = 0;
var oldType;

var charactersRead;

var fileFormat = "txt";

var portCOMnumber;

var configHandle = 0;

var connectAutomatically = 0; //0 - false, 1 - true

var alreadyCreated = false; //if config file was already created > true

var configLenght; //lenght of config file

var maxNumOfRecords = 100;
var currentNumOfRecords = 0;


/*!
 * brief Initialization of application.
 *
 * Funcion calls openConfigFile() which reads configured values from stored config file. Then sets intervals for reading values, check connection and clear fail number. Function enableButtons() enables
 * buttons in GUI which are disabled before connection with MCU is established.
 *
 */
function applicationInit()
{
  openConfigFile();

  if(intervalHandle1 == 0)
  {
    intervalHandle1 = setInterval(readValues, 1000);
  }
  if(intervalHandle2 == 0)
  {
    intervalHandle2 = setInterval(checkConnection, 2000);
  }
  if(intervalHandle3 == 0)
  {
    intervalHandle3 = setInterval(clearFailNumber, 10000);
  }

  enableButtons();
}

/*!
 * brief Finds on which COM port is mapped MCU.
 *
 * Attempts to establish a connection to the COM port from the function argument. If this fails, the function calls itself and tries another COM port number until it passes through the entire port range or a successful connection.
 * If it succeeds, the readTSA function is called.
 *
 * param portNumber Number of COM port which it tries to connect to.
 */
function discoverDevice(portNumber)
{
  var name = "RS232_port";
  name += portNumber.toString();

  document.getElementById("initInformation").innerHTML = "<b style=\"color: blue;\">Finding device on port (" + portNumber + ")...</b>";
  document.getElementById("initInformation2").innerHTML = "<b style=\"color: blue;\">Finding device on port (" + portNumber + ")...</b>";

  pcm.StartComm(name).then(() => {portCOMnumber = portNumber; readTSA(false);})
    .catch((err) => { if(portNumber < 256) {discoverDevice(portNumber + 1);} else if(portNumber >= 256){document.getElementById("initInformation").innerHTML = "<b style=\"color: red;\">Device was not discovered in COM port range 1 - 256!</b>"; document.getElementById("initInformation2").innerHTML = "<b style=\"color: red;\">Device was not discovered in COM port range 1 - 256!</b>";} });
}

/*!
 * brief The function connects to the defined COM port.
 *
 * This function is called from set interval if connection was lost before.
 *
 */
function automaticConnection()
{
  document.getElementById("initInformation").innerHTML = "<b style=\"color: blue;\">Trying automatic connection...</b>";
  document.getElementById("initInformation2").innerHTML = "<b style=\"color: blue;\">Trying automatic connection...</b>";

  var name = "RS232_port";
  name += portCOMnumber.toString();

  pcm.StartComm(name).then(() => {console.log("Communication port open."); clearInterval(intervalHandle4); intervalHandle4 = 0; readTSA(true);})
    .catch((err) => { console.error("Device was not found on COM port", portCOMnumber); document.getElementById("initInformation").innerHTML = "<b style=\"color: red;\">Device was not found on COM port " + portCOMnumber + "</b>"; document.getElementById("initInformation2").innerHTML = "<b style=\"color: red;\">Device was not found on COM port " + portCOMnumber + "</b>";});
}

/*!
 * brief Reads TSA table from MCU.
 *
 * Determines if the connected device supports FreeMASTER by loading a TSA table.
 *
 * param autoconnect Decides if an automatic connection is being attempted. If not, and the TSA table did not load successfully, the discoverDevice() function is called with an argument of the following COM port.
 */
function readTSA(autoconnect)
{
  pcm.ReadTSA().then(response => {console.log(response.data.count + " symbols extracted from TSA table."); checkAppId();})
    .catch((err) => { pcm.StopComm().then(() => {}); console.warn("TSA table couldn't be read."); if(autoconnect){if(intervalHandle4 == 0){intervalHandle4 = setInterval(automaticConnection, 2000);}} if(!autoconnect){discoverDevice(portCOMnumber + 1);}});
}

/*!
 * brief Checks app identification number.
 *
 * Because a large number of microcontrollers that support FreeMASTER can be connected to the computer at the same time, it is necessary to determine which one is intended for this application.
 * This is done by verifying the value of applicationID.
 *
 */
function checkAppId()
{
  var result;
  pcm.ReadUIntVariable("applicationID", 2).then(response => {result = response.data; if(result == 40284){console.log("Communication port opened on COM port " + portCOMnumber + "."); checkConnection(); applicationInit(true); openConfigFile();}})
    .catch((err) => { console.warn("Couldn't read applicationID value."); pcm.StopComm().then(() => {}); discoverDevice(portCOMnumber + 1);});
}

/*!
 * brief Stops communication with MCU.
 *
 * If disconnect button in web GUI is pressed this function is called. Clears all set intervals and all data displayed in page.
 *
 */
function disconnect()
{
  pcm.StopComm().then(() =>
  {
    console.log("Communication port closed.");
    clearInterval(intervalHandle1);
    clearInterval(intervalHandle2);
    clearInterval(intervalHandle3);
    clearInterval(intervalHandle4);

    intervalHandle1 = 0;
    intervalHandle2 = 0;
    intervalHandle3 = 0;
    intervalHandle4 = 0;

    clearPageInformation();
    clearFailNumber();
  })
    .catch((err) => { console.error("Couldn't close communication port."); });
}

/*!
 * brief Starts communication with MCU.
 *
 * If connect button in web GUI is pressed this function is called. Reads nuber of COM port from input in web page and if it is in range 1 - 256 sets portCOMnumber and calls automaticConnection()
 *
 */
function manualConnect()
{
  var input = Math.trunc(document.getElementById('setCOMport').value);

  if((input < 1) || (input > 256))
  {
    alert("COM port number must be set in range 1 - 256!");
    document.getElementById('setCOMport').value = "";
  }
  else
  {
    portCOMnumber = input;
    automaticConnection();
  }
}

/*!
 * brief Enables automatic connection
 *
 * If enable button to automatic connection is pressed this function is called. Sets the application to perform automatic connection at the next initialization.
 *
 * param init If it is called from initialization configuration is not saved.
 */
function autoEnable(init)
{
  connectAutomatically = 1;
  if(!init)
    saveConfig();
  document.getElementById("autoInformation").innerHTML = "<b style=\"color: green;\">Enabled</b>";
  document.getElementById("button8").disabled = true;
  document.getElementById("button8").style.color = "gray";
  document.getElementById("button9").disabled = false;
  document.getElementById("button9").style.color = "black";

  if(intervalHandle1 == 0)
  {
    document.getElementById("button7").disabled = false;
    document.getElementById("button7").style.color = "black";
  }
  else
  {
    document.getElementById("button7").disabled = true;
    document.getElementById("button7").style.color = "gray";
  }
}

/*!
 * brief Disables automatic connection.
 *
 * If disable button to automatic connection is pressed this function is called. Sets the application to not perform automatic connection at the next initialization.
 *
 * param init If it is called from initialization configuration is not saved.
 */
function autoDisable(init)
{
  connectAutomatically = 0;
  if(!init)
    saveConfig();
  document.getElementById("autoInformation").innerHTML = "<b style=\"color: red;\">Disabled</b>";
  document.getElementById("button8").disabled = false;
  document.getElementById("button8").style.color = "black";
  document.getElementById("button9").disabled = true;
  document.getElementById("button9").style.color = "gray";
  document.getElementById("button7").disabled = true;
  document.getElementById("button7").style.color = "gray";
}

/*!
 * brief Clears fail number.
 *
 * This function is called from set interval. failNumber represents number of failed attempts of communication with MCU. In communication, the connection sometimes fails.
 * These events are counted so that the application does not report a connection error in the event of a single outage. If it exceeds the defined value, a connection interruption is reported.
 * After a defined time, this counter is automatically reset by this function called from the interval.
 *
 */
function clearFailNumber()
{
  failNumber = 0;
}

/*!
 * brief Opens config file.
 *
 * Opens config file. If it is successfull calls function processConfigFile()
 *
 */
function openConfigFile()
{
  if(configHandle != 0)
  {
    pcm.LocalFileClose(configHandle);
    configHandle = 0;
  }

  pcm.LocalFileOpen("C:\\Temp\\data\\conf.txt", "r+").then(response => {configHandle = response.data; processConfigFile();})
    .catch((err) => { console.error("Config file can't be opened!"); });

}

/*!
 * brief Reads data from config file.
 *
 * Reads data from config file and sets variables with values ​​from the configuration file.
 *
 */
function processConfigFile()
{
  var configString = "nothing";
  pcm.LocalFileReadString(configHandle, 255, true).then(response =>
    {
      configString = response.data;

      var values = configString.split(";");

      if((parseInt(values[0]) >= 0) && (parseInt(values[0]) <= 110))
      {
        defaultLimit1 = parseInt(values[0]);
      }

      if((parseInt(values[1]) >= 0) && (parseInt(values[1]) <= 110))
      {
        defaultLimit2 = parseInt(values[1]);
      }

      if((parseInt(values[2]) >= 0) && (parseInt(values[2]) <= 110))
      {
        defaultLimit3 = parseInt(values[2]);
      }

      if((parseInt(values[3]) >= 0) && (parseInt(values[3]) <= 110))
      {
        defaultLimit4 = parseInt(values[3]);
      }

      if((parseInt(values[4]) >= 0) && (parseInt(values[4]) <= 10))
      {
        defaultLimit5 = parseInt(values[4]);
      }

      if((values[5] == "txt") || (values[5] == "csv") || (values[5] == "htm"))
      {
        fileFormat = values[5];
      }

      if((parseInt(values[6]) == 0) || (parseInt(values[6]) == 1))
      {
        connectAutomatically = parseInt(values[6]);
      }

      if((parseInt(values[7]) >= 50))
      {
        maxNumOfRecords = parseInt(values[7]);
      }



      if((intervalHandle1 == 0) && (connectAutomatically == 1))
      {
        writeValues(defaultLimit1, defaultLimit2, defaultLimit3, defaultLimit4, defaultLimit5);
        autoEnable(false);
        discoverDevice(1);
      }
      else if ((intervalHandle1 == 0) && (connectAutomatically == 0))
      {
        writeValues(defaultLimit1, defaultLimit2, defaultLimit3, defaultLimit4, defaultLimit5);
        autoDisable(false);
      }
      else
      {
        writeValues(defaultLimit1, defaultLimit2, defaultLimit3, defaultLimit4, defaultLimit5);
        if (alreadyCreated == false)
        {
          createLogFile(fileFormat);
          sendData(true, "");
        }
      }
    })
    .catch((err) =>
    {
      console.warn("Config file couldn't be read. Default limit values will be used.");

      if(intervalHandle1 != 0)
        writeValues(defaultLimit1, defaultLimit2, defaultLimit3, defaultLimit4, defaultLimit5);

      if(alreadyCreated == false)
      {
        createLogFile(fileFormat);
        sendData(true, "");
      }
    });
}

/*!
 * brief Sets time variables to actual time.
 *
 */
function getTime()
{
  var d = new Date();

  years = d.getFullYear();
  months = (d.getMonth() + 1);
  days = d.getDate();
  hours = d.getHours();
  minutes = d.getMinutes();
  seconds = d.getSeconds();
  time = d.getTime();

  if (months < 10) {months = "0" + months;}
  if (days < 10) {days = "0" + days;}
  if (hours < 10) {hours = "0" + hours;}
  if (minutes < 10) {minutes = "0" + minutes;}
  if (seconds < 10) {seconds = "0" + seconds;}
}

/*!
 * brief Creates log file.
 *
 * Creates log file with name in form DD_MM_YYYY__HH-MM-SS
 *
 * param fileFormat Format of created log file (txt, csv, html)
 */
function createLogFile(fileFormat)
{
  getTime();

  var name = "C:\\Temp\\logs\\Temperature log ";
  name += days;
  name += "_";
  name += months;
  name += "_";
  name += years;
  name += "__";
  name += hours;
  name += "-";
  name += minutes;
  name += "-";
  name += seconds;

  switch (fileFormat)
  {
    case "csv": name += ".csv";
      break;
    case "txt": name += ".txt";
      break;
    case "htm": name += ".html";
      break;
    default:    console.error("Wrong fileFormat! Can't create log file.");
  }

  pcm.LocalFileOpen(name, "w").then(response =>
    {
      alreadyCreated = true;

      handle = response.data;

      var readString;

      switch (fileFormat) {
        case "csv":   pcm.LocalFileWriteString(handle, "Date,Time,Temperature 1 [°C],Temperature 2 [°C],Temperature 3 [°C],Temperature 4 [°C],Relay state, Info message\r\n", true).then(response => {});
          break;
        case "txt":   pcm.LocalFileWriteString(handle, "Date\t\tTime\t\tTemperature 1 [°C]\tTemperature 2 [°C]\tTemperature 3 [°C]\tTemperature 4 [°C]\tRelay state\r\n", true).then(response => {});
          break;
        case "htm":   pcm.LocalFileOpen("C:\\Temp\\data\\logPageTemplate.html", "r+").then(response =>
                        {
                          htmlHandle = response.data
                          pcm.LocalFileReadString(htmlHandle, 575, true).then(response =>
                            {
                              readString = response.data;
                              pcm.LocalFileWriteString(handle, readString, true).then(response => {});
                              pcm.LocalFileClose(htmlHandle);
                            });
                        });

          break;
        default:      console.error("Wrong fileFormat! Can't create log file.");
      }
    })
    .catch((err) => { console.error("Couldn't create log file."); });

}

/*!
 * brief Writes record to log file.
 *
 * param fileFormat format of log file
 */
function writeToLogFile(fileFormat)
{
  getTime();

  ++currentNumOfRecords;
  if(currentNumOfRecords > maxNumOfRecords)
  {
    closeFile();
    createLogFile(fileFormat);
    currentNumOfRecords = 0;
  }

  sendData(false, "");

  var mytime = "";
  mytime += hours;
  mytime += ":";
  mytime += minutes;
  mytime += ":";
  mytime += seconds;
  updateGraph(temperature1, temperature2, temperature3, temperature4, mytime);

  var relayStateStr;

  if(relayState == 0)
    relayStateStr = "OFF";
  else if(relayState == 1)
    relayStateStr = "ON";
  else
    relayStateStr = "undefined";

  var text = ""; //sets data type

  if(fileFormat == "csv")
  {
    text += days;
    text += "/";
    text += months;
    text += "/";
    text += years;
    text += ",";
    text += hours;
    text += ":";
    text += minutes;
    text += ":";
    text += seconds;
    text += ",";
    text += temperature1;
    text += ",";
    text += temperature2;
    text += ",";
    text += temperature3;
    text += ",";
    text += temperature4;
    text += ",";
    text += relayStateStr;
    text += "\r\n";
  }

  else if (fileFormat == "txt")
  {
    text += days;
    text += "/";
    text += months;
    text += "/";
    text += years;
    text += "\t";
    text += hours;
    text += ":";
    text += minutes;
    text += ":";
    text += seconds;
    text += "\t";
    text += temperature1;
    text += "\t\t\t";
    text += temperature2;
    text += "\t\t\t";
    text += temperature3;
    text += "\t\t\t";
    text += temperature4;
    text += "\t\t\t";
    text += relayStateStr;
    text += "\r\n";
  }

  else if (fileFormat == "htm")
  {
    text += "\r\n<tr>\r\n";

    text += "<th>";
    text += days;
    text += "/";
    text += months;
    text += "/";
    text += years;
    text += "</th>\r\n";

    text += "<th>";
    text += hours;
    text += ":";
    text += minutes;
    text += ":";
    text += seconds;
    text += "</th>\r\n";

    text += "<th>";
    text += temperature1;
    text += "</th>\r\n";

    text += "<th>";
    text += temperature2;
    text += "</th>\r\n";

    text += "<th>";
    text += temperature3;
    text += "</th>\r\n";

    text += "<th>";
    text += temperature4;
    text += "</th>\r\n";

    text += "<th>";
    text += relayStateStr;
    text += "</th>\r\n";

    text += "</tr>";
  }

  pcm.LocalFileWriteString(handle, text, true).then(response => {console.log("Successfully written to log file.")})
    .catch((err) => { console.error("Couldn't write row to log file."); });

  old_old_temperature1 = old_temperature1;
  old_old_temperature2 = old_temperature2;
  old_old_temperature3 = old_temperature3;
  old_old_temperature4 = old_temperature4;

  old_temperature1 = temperature1;
  old_temperature2 = temperature2;
  old_temperature3 = temperature3;
  old_temperature4 = temperature4;

  old_relayState = relayState;
}


var request;

/*!
 * brief Sends data to Google Sheets.
 *
 * Sends data to Google Sheets where it is processed by script on Google Apps Script. Data is sent using POST method.
 *
 * param startFlag determines if this is new log file, if yes data is send with x value wich is processed by script on Google Sheets that it creates a new sheet
 * param infoMessage message that adds information about what caused relay disconnection
 */
function sendData(startFlag, infoMessage)
{
  // Abort any pending request
  if (request) {
      request.abort();
  }

  // Serialize the data in the form
  var serializedData;

  if (startFlag === true)
  {
    serializedData = "date=";
    serializedData += "x";
    serializedData += "&time=";
    serializedData += "x";
    serializedData += "&temperature_1=";
    serializedData += "x";
    serializedData += "&temperature_2=";
    serializedData += "x";
    serializedData += "&temperature_3=";
    serializedData += "x";
    serializedData += "&temperature_4=";
    serializedData += "x";
    serializedData += "&relay=";
    serializedData += "x";
    serializedData += "&Info_message=";
    serializedData += "x";
  }

  else
  {
    serializedData = "date=";
    serializedData += days;
    serializedData += "/";
    serializedData += months;
    serializedData += "/";
    serializedData += years;
    serializedData += "&time=";
    serializedData += hours;
    serializedData += ":";
    serializedData += minutes;
    serializedData += ":";
    serializedData += seconds;
    serializedData += "&temperature_1=";
    serializedData += temperature1;
    serializedData += "&temperature_2=";
    serializedData += temperature2;
    serializedData += "&temperature_3=";
    serializedData += temperature3;
    serializedData += "&temperature_4=";
    serializedData += temperature4;

    serializedData += "&relay=";
    var stateOfRelay;
    if (relayState == 1)
      stateOfRelay = "ON";
    else
      stateOfRelay = "OFF";

    serializedData += stateOfRelay;

    serializedData += "&Info_message=";
    serializedData += infoMessage;
  }

  // Fire off the request to /form.php
  request = $.ajax({
      url: "https://script.google.com/macros/s/AKfycbwj2A48kwoJ1iilC2PgZ_6TI50rt01uZHQJVuohHbRRXTi2u94/exec",
      type: "post",
      data: serializedData
  });

  // Callback handler that will be called on success
  request.done(function (response, textStatus, jqXHR){
      // Log a message to the console
      console.log("Data sent to Google");
      //console.log(response);
      //console.log(textStatus);
      //console.log(jqXHR);
  });

  // Callback handler that will be called on failure
  request.fail(function (jqXHR, textStatus, errorThrown){
      // Log the error to the console
      console.error(
          "The following error occurred: "+
          textStatus, errorThrown
      );
  });

  // Callback handler that will be called regardless
  // if the request failed or succeeded
  request.always(function () {
  });

}

/*!
 * brief Reads measured value.
 *
 * Reads measured value. Then it stores value in variable and calls function setColor() which sets color of read value in web page depending on whether it exceeded the limit value.
 *
 * param symbolName name of value which will be read from MCU
 * param htmlElement name of element in web page which is set to read value from MCU
 * param limitValue limit value of measured value
 */
function readMeasuredValue(symbolName, htmlElement, limitValue)
{
  var result;
  pcm.ReadDoubleVariable(symbolName).then(response =>
    {
      result = response.data.toFixed(1);
      document.getElementById(htmlElement).innerHTML = result;
      setColor(htmlElement, result, limitValue);

      result = parseInt(result);

      switch (symbolName)
      {
        case "temperature1": temperature1 = result;
          break;
        case "temperature2": temperature2 = result;
          break;
        case "temperature3": temperature3 = result;
          break;
        case "temperature4": temperature4 = result;
          break;
        case "current": current = result;
          break;
      }
    })
    .catch((err) => { console.warn("Couldn't read measured value."); });
}

/*!
 * brief Reads limit value.
 *
 * param symbolName name of value which will be read from MCU
 * param htmlElement name of element in web page which is set to read value from MCU
 */
function readLimit(symbolName, htmlElement)
{
  var result;
  pcm.ReadDoubleVariable(symbolName).then(response => {result = response.data; document.getElementById(htmlElement).innerHTML = result;})
    .catch((err) => { console.warn("Couldn't read limit value."); });
}

/*!
 * brief Function calls functions to read all values from MCU.
 *
 * Function calls functions to read all values from MCU. Then it updates items of webpage with load data. If read temperature or current values is different from previous values function writeToLogFile() is called.
 * If disconnectionSrc is different from previous value new infoMessage is written to log file.
 *
 */
function readValues()
{
  readMeasuredValue("temperature1", "value1", lim1);
  readMeasuredValue("temperature2", "value2", lim2);
  readMeasuredValue("temperature3", "value3", lim3);
  readMeasuredValue("temperature4", "value4", lim4);
  readMeasuredValue("current", "value5", lim5);

  pcm.ReadUIntVariable("relayState", 2).then(response =>
    {
      relayState = response.data;

      if(relayState == 0)
      {
        document.getElementById("state").innerHTML = "OFF";
        document.getElementById("state2").innerHTML = "OFF";
        document.getElementById("state").style.color = "red";
        document.getElementById("state2").style.color = "red";

        document.getElementById("button3").disabled = false;
        document.getElementById("button3").style.color = "black";
        document.getElementById("button4").disabled = true;
        document.getElementById("button4").style.color = "gray";
      }
      else if(relayState == 1)
      {
        document.getElementById("state").innerHTML = "ON";
        document.getElementById("state2").innerHTML = "ON";
        document.getElementById("state").style.color = "green";
        document.getElementById("state2").style.color = "green";

        document.getElementById("button3").disabled = true;
        document.getElementById("button3").style.color = "gray";
        document.getElementById("button4").disabled = false;
        document.getElementById("button4").style.color = "black";
      }
      else
      {
        document.getElementById("state").innerHTML = "undefined";
        document.getElementById("state2").innerHTML = "undefined";
        document.getElementById("state").style.color = "gray";
        document.getElementById("state2").style.color = "gray";
      }
    })
    .catch((err) => { console.warn("Couldn't read relayState value."); });

  pcm.ReadUIntVariable("disconnectionSrc", 2).then(response =>
    {
      disconnectionSrc = response.data;

      switch (disconnectionSrc)
      {
        case 0: document.getElementById("commentState").innerHTML = "";
                document.getElementById("commentState2").innerHTML = "";
          break;
        case 1: document.getElementById("commentState").innerHTML = " - Temperature 1 has exceeded the limit";
                document.getElementById("commentState2").innerHTML = " - Temperature 1 has exceeded the limit";
          break;
        case 2: document.getElementById("commentState").innerHTML = " - Temperature 2 has exceeded the limit";
                document.getElementById("commentState2").innerHTML = " - Temperature 2 has exceeded the limit";
          break;
        case 3: document.getElementById("commentState").innerHTML = " - Temperature 3 has exceeded the limit";
                document.getElementById("commentState2").innerHTML = " - Temperature 3 has exceeded the limit";
          break;
        case 4: document.getElementById("commentState").innerHTML = " - Temperature 4 has exceeded the limit";
                document.getElementById("commentState2").innerHTML = " - Temperature 4 has exceeded the limit";
          break;
        case 5: document.getElementById("commentState").innerHTML = " - Current has exceeded the limit";
                document.getElementById("commentState2").innerHTML = " - Current has exceeded the limit";
          break;
        case 6: document.getElementById("commentState").innerHTML = " - Manual disconnection";
                document.getElementById("commentState2").innerHTML = " - Manual disconnection";
          break;
        default: document.getElementById("commentState").innerHTML = "";
                 document.getElementById("commentState2").innerHTML = "";
      }

      if (disconnectionSrc != old_disconnectionSrc)
      {
        writeInfoMessage(disconnectionSrc);
        old_disconnectionSrc = disconnectionSrc;
      }

    })
    .catch((err) => { console.warn("Couldn't read disconnectionSrc value."); });

  if((temperature1 != undefined) && (temperature2 != undefined) && (temperature3 != undefined) && (temperature4 != undefined) && (current != undefined) && (relayState != undefined))
  {
    if(((temperature1 != old_temperature1) && (temperature1 != old_old_temperature1)) || ((temperature2 != old_temperature2) && (temperature2 != old_old_temperature2)) || ((temperature3 != old_temperature3) && (temperature3 != old_old_temperature3)) || ((temperature4 != old_temperature4) && (temperature4 != old_old_temperature4)) || (relayState != old_relayState))
    {
      writeToLogFile(fileFormat);
    }
  }

  pcm.ReadUIntVariable("resetState", 2).then(response =>
    {
      var resetState = response.data;

      if((resetState == 1) && (lim1 != undefined) && (lim2 != undefined) && (lim3 != undefined) && (lim4 != undefined) && (lim5 != undefined))
      {
        writeValues(lim1, lim2, lim3, lim4, lim5);

        pcm.WriteUIntVariable("resetState", 2, 0).then(() => {})
          .catch((err) => { console.error("Reset state couldn't be set to false."); });
      }

    })
    .catch((err) => { console.warn("Couldn't read resetState value."); });

  readLimit("limit_temperature1", "limit1");
  readLimit("limit_temperature2", "limit2");
  readLimit("limit_temperature3", "limit3");
  readLimit("limit_temperature4", "limit4");
  readLimit("limit_current", "limit5");

}

/*!
 * brief Sets color of measured data in webpage.
 *
 * Sets color of measured data in webpage. If measured value is between 80 and 90% of limit value color is set to orange. If it is over 90% color is set to red.
 *
 * param elementName name of set HTML element
 * param measuredValue measured value
 * param limitValue limit value
 */
function setColor(elementName, measuredValue, limitValue)
{
 if((measuredValue > (0.80 * limitValue)) && (measuredValue < (0.90 * limitValue)))
 {
   document.getElementById(elementName).style.color = "orange";
 }
 else if (measuredValue > (0.90 * limitValue))
 {
   document.getElementById(elementName).style.color = "red";
 }
 else
 {
   document.getElementById(elementName).style.color = "black";
 }
}

/*!
 * brief Writes limit values to MCU.
 *
 * Writes limit values to MCU and config file. Before these values are written they are checked if not exceeds maximum permissible limit values.
 *
 * param limit1 limit of temperature1
 * param limit1 limit of temperature2
 * param limit1 limit of temperature3
 * param limit1 limit of temperature4
 * param limit1 limit of current
 */
function writeValues(limit1, limit2, limit3, limit4, limit5)
{
  if((limit1 > 110) || (limit2 > 110) || (limit3 > 110) || (limit4 > 110) || (limit5 > 10))
  {
    console.warn("The entered limit values ​​have exceeded the maximum allowable limit!");
  }
  else if ((limit1 < 0) || (limit2 < 0) || (limit3 < 0) || (limit4 < 0) || (limit5 < 0))
  {
    console.warn("The entered limit values ​​are below the minimum allowable limit!");
  }
  else
  {
    lim1 = limit1;
    lim2 = limit2;
    lim3 = limit3;
    lim4 = limit4;
    lim5 = limit5;

    if(intervalHandle1 != 0)
    {
      pcm.WriteDoubleVariable("limit_temperature1", limit1).then(() => {})
        .catch((err) => { console.error("Couldn't write limit1 to MCU."); });
      pcm.WriteDoubleVariable("limit_temperature2", limit2).then(() => {})
        .catch((err) => { console.error("Couldn't write limit2 to MCU."); });
      pcm.WriteDoubleVariable("limit_temperature3", limit3).then(() => {})
        .catch((err) => { console.error("Couldn't write limit3 to MCU."); });
      pcm.WriteDoubleVariable("limit_temperature4", limit4).then(() => {})
        .catch((err) => { console.error("Couldn't write limit4 to MCU."); });
      pcm.WriteDoubleVariable("limit_current", limit5).then(() => {})
        .catch((err) => { console.error("Couldn't write limit5 to MCU."); });
    }

    saveConfig();
  }
}

/*!
 * brief Sets new limit values.
 *
 * Sets new limit values. This function is called from GUI using button set limits.
 * It checks if limit values are in permissible range if not warning is displayed.
 * If they are in permissible range new limit values are stored and written to MCU using function writeValues().
 *
 */
function setLimits()
{
  var l1 = Math.round(document.getElementById('setLimit1').value);
  var l2 = Math.round(document.getElementById('setLimit2').value);
  var l3 = Math.round(document.getElementById('setLimit3').value);
  var l4 = Math.round(document.getElementById('setLimit4').value);
  var l5 = Math.round(document.getElementById('setLimit5').value);

  var warnString = "Limits must be set in allowed range!\r\n";
  var overLimits = false;

  if((l1 < 0) || (l1 > 110))
  {
    warnString += "Limit temperature1 couldn't be se to " + l1 + " °C. It must be in range (0 °C - 110 °C)\r\n";
    overLimits = true;
    l1 = lim1;
  }
  if((l2 < 0) || (l2 > 110))
  {
    warnString += "Limit temperature2 couldn't be se to " + l2 + " °C. It must be in range (0 °C - 110 °C)\r\n";
    overLimits = true;
    l2 = lim2;
  }
  if((l3 < 0) || (l3 > 110))
  {
    warnString += "Limit temperature3 couldn't be se to " + l3 + " °C. It must be in range (0 °C - 110 °C)\r\n";
    overLimits = true;
    l3 = lim3;
  }
  if((l4 < 0) || (l4 > 110))
  {
    warnString += "Limit temperature4 couldn't be se to " + l4 + " °C. It must be in range (0 °C - 110 °C)\r\n";
    overLimits = true;
    l4 = lim4;
  }
  if((l5 < 0) || (l5 > 10))
  {
    warnString += "Limit current couldn't be se to " + l5 + " A. It must be in range (0 A - 10 A)\r\n";
    overLimits = true;
    l5 = lim5;
  }

  if(overLimits)
  {
    alert(warnString);
  }

  document.getElementById("setLimit1").value = l1;
  document.getElementById("setLimit2").value = l2;
  document.getElementById("setLimit3").value = l3;
  document.getElementById("setLimit4").value = l4;
  document.getElementById("setLimit5").value = l5;

  writeValues(l1, l2, l3, l4, l5);
}

/*!
 * brief Saves config file.
 *
 * Saves config file in defined format: limit1;limit2;limit3;limit4;limit5;fileFormat;connectAutomatically;maxNumOfRecords;
 *
 */
function saveConfig()
{

  if(configHandle != 0)
  {
    pcm.LocalFileClose(configHandle);
    configHandle = 0;
  }

  pcm.LocalFileOpen("C:\\Temp\\data\\conf.txt", "r+").then(response =>
    {
      configHandle = response.data;
      string = lim1 + ";" + lim2 + ";" + lim3 + ";" + lim4 + ";" + lim5 + ";" + fileFormat + ";" + connectAutomatically + ";" + maxNumOfRecords + ";";

      pcm.LocalFileWriteString(configHandle, string, true).then(response => {  })
        .catch((err) => { console.error("Config file couldn't be saved."); });
      pcm.LocalFileClose(configHandle);
      configHandle = 0;
    })
    .catch((err) => { console.error("Config file couldn't be opened - configuration is not saved."); });
}

/*!
 * brief Sets file type.
 *
 * Sets file type. This function is called from GUI after pressing button set file type. It saves config with new file type, closes an already open file and creates new config file with set format.
 *
 */
function setFileType()
{
  var e = document.getElementById("fileType");
	var result = e.options[e.selectedIndex].value;

  saveConfig();
  closeFile();

  fileFormat = result;
  createLogFile(fileFormat);
}

/*!
 * brief Sets number of records.
 *
 * Sets number of records. This function is called from GUI after pressing button set number. Is sets maximum number of records created per log file and saves this number to config file.
 *
 */
function setNumOfRecords()
{
  maxNumOfRecords = document.getElementById("numOfRecords").value;
  saveConfig();
}

/*!
 * brief Connects relay.
 *
 * Connects relay if limit values are not exceeded.
 *
 */
function connectRelay()
{
  if((temperature1 > lim1) || (temperature2 > lim2) || (temperature3 > lim3) || (temperature4 > lim4) || (current > lim5))
  {
    var warnString = "The measured values exceed the limit values:\r\n";

    if (temperature1 > lim1)
      warnString += "temperature1 (" + temperature1 + " °C)\r\n";
    if (temperature2 > lim2)
      warnString += "temperature2 (" + temperature2 + " °C)\r\n";
    if (temperature3 > lim3)
      warnString += "temperature3 (" + temperature3 + " °C)\r\n";
    if (temperature4 > lim4)
      warnString += "temperature4 (" + temperature4 + " °C)\r\n";
    if (current > lim5)
      warnString += "current (" + current + " A)\r\n";

    warnString += "\r\nThe relay will not be switched on.";

    alert(warnString);
  }
  else
  {
    pcm.WriteUIntVariable("relayState", 1, 1).then(() => {})
      .catch((err) => { console.error("Relay state couldn't be set to ON"); });
  }
}

/*!
 * brief Disconnects relay.
 *
 */
function disconnectRellay()
{
  pcm.WriteUIntVariable("relayState", 1, 0).then(() => {})
    .catch((err) => { console.error("Relay state couldn't be set to OFF"); });
}

/*!
 * brief Counts number of failed communication attempts.
 *
 * Counts number of failed communication attempts.
 * If three and more attempts fails web application goes into state "Target disconnect", intervals are cleared, page informations are cleard and automaticConnection interval, which tries to reestablish connection is set.
 *
 */
function lostConnection()
{
  failNumber += 1;

  if(failNumber == 2)
  {
    document.getElementById("initInformation").innerHTML = "<b style=\"color: orange;\">Lost connection</b>";
    document.getElementById("initInformation2").innerHTML = "<b style=\"color: orange;\">Lost connection</b>";
    writeInfoMessage(7);
  }

  if(failNumber > 2)
  {
    clearInterval(intervalHandle1);
    intervalHandle1 = 0;
    clearInterval(intervalHandle2);
    intervalHandle2 = 0;
    clearInterval(intervalHandle3);
    intervalHandle3 = 0;
    clearPageInformation();
    clearFailNumber();
    writeInfoMessage(8);

    if(intervalHandle4 == 0) //if intervalHandle4 was not set before
    {
      intervalHandle4 = setInterval(automaticConnection, 2000);
    }
  }
}

/*!
 * brief Checks connection with MCU.
 *
 * Checks connection with MCU. Random number is written to MCU then is read using checkResponse.
 *
 */
function checkConnection()
{
  var writtenValue = Math.floor(Math.random() * 10000); //random number generation between 0 - 9999
  pcm.WriteUIntVariable("testValue", 2, writtenValue).then(() => {checkResponse(writtenValue);})
    .catch((err) => {console.warn("checkConnection(): Connection failed!"); lostConnection();});
}

/*!
 * brief Reads written value on MCU.
 *
 * Reads written value on MCU. If it is successful then calls function checkValue which compares written and read value.
 *
 * param writtenValue value written to MCU by checkConnection() function
 */
function checkResponse(writtenValue)
{
  var readValue;
  pcm.ReadUIntVariable("testValue", 2).then(response => {readValue = response.data; checkValue(readValue, writtenValue);})
    .catch((err) => {console.warn("checkConnection(): Connection failed!"); lostConnection();});
}

/*!
 * brief Compares written and read value.
 *
 * Compares written and read value. If they are not the same lostConnection() is called
 *
 * param readValue value read from MCU by checkResponse() function
 * param writtenValue value written to MCU by checkConnection() function
 */
function checkValue(readValue, writtenValue)
{
  if(readValue == writtenValue)
  {
    document.getElementById("initInformation").innerHTML = "<b style=\"color: green;\">Application is running. (COM port " + portCOMnumber + ")</b>";
    document.getElementById("initInformation2").innerHTML = "<b style=\"color: green;\">Application is running. (COM port " + portCOMnumber + ")</b>";
    writeInfoMessage(9);
  }
  else
  {
    lostConnection();
  }
}

/*!
 * brief Clears page information.
 *
 * Clears page information. Function is called after initialization or after disconnection of target.
 *
 */
function clearPageInformation()
{
  document.getElementById("value1").innerHTML = "N/A";
  document.getElementById("value1").style.color = "black";
  document.getElementById("value2").innerHTML = "N/A";
  document.getElementById("value2").style.color = "black";
  document.getElementById("value3").innerHTML = "N/A";
  document.getElementById("value3").style.color = "black";
  document.getElementById("value4").innerHTML = "N/A";
  document.getElementById("value4").style.color = "black";
  document.getElementById("value5").innerHTML = "N/A";
  document.getElementById("value5").style.color = "black";

  document.getElementById("limit1").innerHTML = "N/A";
  document.getElementById("limit1").style.color = "black";
  document.getElementById("limit2").innerHTML = "N/A";
  document.getElementById("limit2").style.color = "black";
  document.getElementById("limit3").innerHTML = "N/A";
  document.getElementById("limit3").style.color = "black";
  document.getElementById("limit4").innerHTML = "N/A";
  document.getElementById("limit4").style.color = "black";
  document.getElementById("limit5").innerHTML = "N/A";
  document.getElementById("limit5").style.color = "black";

  lim1 = undefined;
  lim2 = undefined;
  lim3 = undefined;
  lim4 = undefined;
  lim5 = undefined;

  document.getElementById("state").innerHTML = "N/A";
  document.getElementById("state").style.color = "black";
  document.getElementById("state2").innerHTML = "N/A";
  document.getElementById("state2").style.color = "black";

  document.getElementById("commentState").innerHTML = "";
  document.getElementById("commentState2").innerHTML = "";

  document.getElementById("button2").disabled = true;
  document.getElementById("button2").style.color = "gray";
  document.getElementById("button3").disabled = true;
  document.getElementById("button3").style.color = "gray";
  document.getElementById("button4").disabled = true;
  document.getElementById("button4").style.color = "gray";

  document.getElementById("button6").disabled = true; //disable disconnect
  document.getElementById("button6").style.color = "gray";
  document.getElementById("button5").disabled = false; //enable connect
  document.getElementById("button5").style.color = "black";
  document.getElementById("setCOMport").disabled = false; //enable COM port input

  if(connectAutomatically == 1)
  {
    document.getElementById("button7").disabled = false; //disable disconnect
    document.getElementById("button7").style.color = "black";
  }

  if(connectAutomatically == 0)
  {
    document.getElementById("button7").disabled = true; //disable disconnect
    document.getElementById("button7").style.color = "gray";
  }

  document.getElementById("initInformation").innerHTML = "<b style=\"color: red;\">Target disconnected</b>";
  document.getElementById("initInformation2").innerHTML = "<b style=\"color: red;\">Target disconnected</b>";
}

/*!
 * brief Enables buttons in webpage.
 *
 * Enables buttons in webpage. Buttons that trigger functions that require connection to the MCU are disabled when disconnected.
 *
 */
function enableButtons()
{
  document.getElementById("button2").disabled = false;
  document.getElementById("button2").style.color = "black";

  document.getElementById("button6").disabled = false; //enable disconnect
  document.getElementById("button6").style.color = "black";
  document.getElementById("button5").disabled = true; //disable connect
  document.getElementById("button5").style.color = "gray";
  document.getElementById("setCOMport").disabled = true; //disable COM port input
  document.getElementById("setCOMport").value = portCOMnumber;

  if(connectAutomatically == 1)
    autoEnable(true);
  if(connectAutomatically == 0)
    autoDisable(true);
}

/*!
 * brief Writes info message to log file.
 *
 * param type type of message written to log file
 */
function writeInfoMessage(type)
{
  if((type < 7) || ((type > 6) && (type != oldType)))
  {
    getTime();

    var relayStateStr;

    if(relayState == 1)
      relayStateStr = "ON";
    else
      relayStateStr = "OFF";

    var infoMessage = "";
    var text = "";

    if(fileFormat == "csv")
    {
      text += days;
      text += "/";
      text += months;
      text += "/";
      text += years;
      text += ",";
      text += hours;
      text += ":";
      text += minutes;
      text += ":";
      text += seconds;
      text += ",";
      text += temperature1;
      text += ",";
      text += temperature2;
      text += ",";
      text += temperature3;
      text += ",";
      text += temperature4;
      text += ",";
      text += relayStateStr;
      text += ",";
    }
    else if(fileFormat == "txt")
    {
      text += days;
      text += "/";
      text += months;
      text += "/";
      text += years;
      text += "\t";
      text += hours;
      text += ":";
      text += minutes;
      text += ":";
      text += seconds;
      text += "\t";
    }
    else if(fileFormat == "htm")
    {
      text += "\r\n<tr>\r\n";

      text += "<th>";
      text += days;
      text += "/";
      text += months;
      text += "/";
      text += years;
      text += "</th>\r\n";

      text += "<th>";
      text += hours;
      text += ":";
      text += minutes;
      text += ":";
      text += seconds;
      text += "</th>\r\n";

      text += "<td colspan=\"5\">";
    }

    switch (type)
    {
      case 0: infoMessage = "[ON] Relay has been manually connected.";
        break;
      case 1: infoMessage = "[OFF] Temperature 1 has exceeded the limit - the relay has been disconnected. (Measured value: " + temperature1 + " °C, Limit value: " + limit_temperature1 + " °C)";
        break;
      case 2: infoMessage = "[OFF] Temperature 2 has exceeded the limit - the relay has been disconnected. (Measured value: " + temperature2 + " °C, Limit value: " + limit_temperature2 + " °C)";
        break;
      case 3: infoMessage = "[OFF] Temperature 3 has exceeded the limit - the relay has been disconnected. (Measured value: " + temperature3 + " °C, Limit value: " + limit_temperature3 + " °C)";
        break;
      case 4: infoMessage = "[OFF] Temperature 4 has exceeded the limit - the relay has been disconnected. (Measured value: " + temperature4 + " °C, Limit value: " + limit_temperature4 + " °C)";
        break;
      case 5: infoMessage = "[OFF] Current has exceeded the limit - the relay has been disconnected. (Measured value: " + current + " A, Limit value: " + limit_current + " A)";
        break;
      case 6: infoMessage = "[OFF] Relay has been manually disconnected.";
        break;
      case 7: infoMessage = "[KO] checkConnection(): Lost connection!";
        break;
      case 8: infoMessage = "[KO] checkConnection(): Target disconnected!";
        break;
      case 9: infoMessage = "[OK] checkConnection(): Connection reestablished.";
        break;
    }

    text += infoMessage;
    text += "\r\n";

    if(fileFormat == "htm")
    {
      text += "</td></tr>\r\n";
    }

    sendData(false, infoMessage);

    if(handle != 0) //if log file is opened
    {
      pcm.LocalFileWriteString(handle, text, true);
    }

    if(type > 6)
    {
      oldType = type;
    }
  }
}

/*!
 * brief Closes opened log file.
 *
 * Closes opened log file. If opened log file is html file closing tags are added to file.
 *
 */
function closeFile()
{
  if(fileFormat == "htm")
  {
    pcm.LocalFileWriteString(handle, "\r\n\r\n</table>\r\n\r\n</body>\r\n</html>", true).then(response => {})
      .catch((err) => { console.error("Couldn't write end of html log file."); });
  }

  pcm.LocalFileClose(handle);
  handle = 0;
}

/*!
 * brief Shows settings.
 *
 * Shows settings. This function is called after pressing button to show settings in webpage.
 *
 */
function showSettings()
{
  document.getElementById("settingsOverlay").style.display = "block";
  document.getElementById("fileType").value = fileFormat;
  document.getElementById("numOfRecords").value = maxNumOfRecords;
  document.getElementById("setLimit1").value = lim1;
  document.getElementById("setLimit2").value = lim2;
  document.getElementById("setLimit3").value = lim3;
  document.getElementById("setLimit4").value = lim4;
  document.getElementById("setLimit5").value = lim5;
}

/*!
 * brief Hides settings.
 *
 * Hide settings. This function is called after pressing button to hide settings in webpage.
 *
 */
function hideSettings()
{
  document.getElementById("settingsOverlay").style.display = "none";
}

/*!
 * brief Shows connection.
 *
 * Shows connection. This function is called after pressing button to show connection settings in webpage.
 *
 */
function showConnection()
{
  document.getElementById("connectionOverlay").style.display = "block";
  document.getElementById("setCOMport").value = portCOMnumber;
}

/*!
 * brief Hides connection.
 *
 * Hide connection. This function is called after pressing button to hide connection settings in webpage.
 *
 */
function hideConnection()
{
  document.getElementById("connectionOverlay").style.display = "none";
  document.getElementById("fileType").value = fileFormat;
  document.getElementById("numOfRecords").value = maxNumOfRecords;
  document.getElementById("setLimit1").value = lim1;
  document.getElementById("setLimit2").value = lim2;
  document.getElementById("setLimit3").value = lim3;
  document.getElementById("setLimit4").value = lim4;
  document.getElementById("setLimit5").value = lim5;
}

/*!
 * brief Closes opened files if webpage is closed.
 *
 * Closes opened files if webpage is closed. This function is called automatically by browser when the webpage is closed.
 *
 */
window.onbeforeunload = function()
{
  if(fileFormat == "htm")
  {
    pcm.LocalFileWriteString(handle, "\r\n\r\n</table>\r\n\r\n</body>\r\n</html>", true).then(response => {})
      .catch((err) => { console.error("Couldn't write end of html log file."); });
  }

  if(handle != 0)
  {
    pcm.LocalFileClose(handle);
    handle = 0;
  }

  if(configHandle != 0)
  {
    pcm.LocalFileClose(configHandle);
    configHandle = 0;
  }

}
